##
## Sample1.py - Line creation sample
##
"""Line creation sample.

This small python script demonstrates the creation of some CADdy++ 2D lines.
Additionally, the sample shows different ways CAD data types can be build.
"""
from CADdy.Application import GetApplic
from CADdy.Utility.Cast import Child, Parent
from CADdy.Utility.Model import AddToParent

def main():
    theCADdy = GetApplic()

    ##
    ## Manager objects from the application object. This object provides
    ## convenience access to the multipel CADdy++ manager objects.
    ##
    factory       = theCADdy.Factory2D()
    targetManager = theCADdy.TargetManager()

    ##
    ## Access the current target group/sketchboard from the target manager.
    ##
    parent = targetManager.TargetOp2DParent
    
    ##
    ## First, creation of instances from registered COM components 
    ## via the dispatch interface.
    ##
    pos1 = theCADdy.CreateObject('CADdyDataTypes.CADdy2DPosition')
    pos2 = theCADdy.CreateObject('CADdyDataTypes.CADdy2DPosition')
    
    pos1.SetCoord(100.0, 100.0)
    pos2.SetCoord(200.0, 200.0)
    
    ##
    ## Second, using the CADdyDataTypes COM object, providing CoClasses
    ## which allows the creation of low level CAD data types.
    ##
    pos3 = theCADdy.DataTypes().CADdy2DPosition()
    pos4 = theCADdy.DataTypes().CADdy2DPosition()
    
    pos3.SetCoord(200.0, 200.0)
    pos4.SetCoord(300.0, 100.0)
    
    ##
    ## Third, using the CADdyDataTypes COM object's data type manager,
    ## which provids a generational interface for the low level CAD
    ## data types.
    ##
    pos5 = theCADdy.DataTypesManager().Create2DPosition(300, 100)
    pos6 = theCADdy.DataTypesManager().Create2DPosition(400, 200)
    
    ##
    ## Create lines. The 2D factory manager allows the creation of high
    ## level CAD objects. Additionaly, there exists a 3D factory manager.
    ##
    line1 = factory.CreateOp2DStraSeg(pos1,pos2)
    line2 = factory.CreateOp2DStraSeg(pos3,pos4)
    line3 = factory.CreateOp2DStraSeg(pos5,pos6)

    ##
    ## Add the lines into the target group/sketchboard. The ICAdyChild and 
    ## ICADdyParent interfaces provide the necessary functionality. So, we 
    ## must cast to these interfaces.
    ##
    child1  = theCADdy.Model().ICADdyChild(line1)
    child2  = theCADdy.Model().ICADdyChild(line2)
    
    parent = theCADdy.Model().ICADdyParent(parent)

    parent.Add(child1)
    parent.Add(child2)
    
    ##
    ## Another, convenience way is the usage of a Cast utility module provided by the
    ## framework which ease the work with the CADdy++ COM interface.
    ##
    parent = Parent(parent)
    child2 = Child(line2)
    
    parent.Add(child2)
    
    ##
    ## Or direct use of the tool's AddToParent function from the Model utility module.
    ##
    AddToParent(line3, targetManager.TargetOp2DParent)
    

if __name__ == '__main__':
    main()
