##
## AddIn1.py - More lines...
##
"""AddIn sample 1: Command pool and command generation example.

This sample series (0-3) of python scripts introduces the way CADdy++ 
AddIns should be written.

An 'AddIn' is a kind of plugin. Actually, it is a registered COM server which 
provides arbitrarily services, which may be meaningful for the CADdy++ user. 

Registration of the AddIn can be performed by just running this script
from the command shell. Deregistration can be performed by running the
script this way: 'AddIn1.py --unregister'

This example shows the generation of a CADdy++ 'Command Pool' 'AddIn1', which
contains a CADdy++ 'Command' 'MyCmd1'. Executing this command will draw just
a bunch of lines. A commad pool can contain multipe commands.

Starting of the AddIn1 will be performed by running of the associated 
script RunAddIn1.py. After that the command pool 'AddIn1' containing
the command 'MyCmd1' will be established. Command 'AddIn1.MyCmd1' can be
started by associating a CADdy++ tool bar button with it. For that
look into the CADdy++ user documentation or into the CADdy++ application
menu 'View/Symbol Bar'. 

Sample AddIn1 shows the most simple usage pattern of the CADdy++ AddIn framework.
On exectuting the AddIn, function OnInit() is called with the application object
as the only function parameter. The AddIn1 implements the creation of a bunch of
lines in the OnInit handler.
"""
from CADdy.AddInServer   import *
from CADdy.Utility.Model import AddToParent
from win32com.client     import constants

##
## The AddIn Server
##
class AddIn1(AddInBase):
    """AddIn1 - class derived from convenience class AddInBase.
    
    The following two characteristics must be provided on the definition of 
    an AddIn:
        _reg_clsid_  : A GUID string by which the AddIn is uniquely referenced.
                       The CADdy++ Python COM framework provides two convenience
                       methods for generating such a GUID string.
                       Firstly, from within the Python interactive interpreter,
                       and secondly from a DOS command shell.
                       1. The CADdy.AddInServer module provides a method Guid(),
                          which prints a new GUID string to the standard output.
                          Example:
                            >>> from CADdy.AddInServer import Guid
                            >>> Guid()
                            IID('{10907304-168F-492F-91E6-1B813ACC0027}')
                       2. From a DOS shell the modul AddInServer.py in folder
                          CADdy can be directly called. It just prints a newly
                          created GUID and returns.
                          Example:
                            > cd {CADdyInstallPath}\Program\Python\Lib\site-packages\CADdy
                            > AddInServer.py
                            {10907304-168F-492F-91E6-1B813ACC0027}
        _reg_progid_ : The name of the AddIn.
    """
    _reg_clsid_  = '{10907304-168F-492F-91E6-1B813ACC0027}'
    _reg_progid_ = 'CADdyAddIn.PythonAddIn1'

    def __init__(self):
        """Constructor - just call the base constructor."""
        AddInBase.__init__(self)

    def OnInit(self, theCADdy):
        """Main entry of the AddIn. 
        
        This function is called when the AddIn is invoked. The AddIn implementation
        goes here. We just create a CmdPool object and hand over the application
        object to the constructor.
        """
        self.cmdPool = CmdPool(theCADdy)
        
    def OnExit(self, theCADdy):
        """Main exit point of the AddIn."""
        self.cmdPool.Delete()


##
## The Command Pool Representation
##
class CmdPool(CmdPoolBase):
    """CmdPool - class derived from convenience class CmdPoolBase.
    
    An instance of this class adds a command pool 'AddIn1' to the CADdy++ root 
    command pool. Then it adds a command 'MyCmd1' which has to be connected with a
    event handler. The event handler is an instance of a class derived from the
    convenience class CmdEvtHandlerBase.
    """
    def __init__(self, theCADdy):
        """Constructor - takes the application object.
        
        The functions AddPool and AddCmd allow the generation of CADdy++ command pools
        and of CADdy++ commands. Multiple pools and commands can be generated. Special
        constants like CMD_NOBREAK are used to define the behaviour with respect to
        already running commands during execution of the command.
        The AddCmd function takes a meta class object derived from CmdEvtHandlerBase. 
        On execution of the command a CmdEvtHandler instance is created and its 
        OnExecute function is called.
        """
        CmdPoolBase.__init__(self, theCADdy)

        newPool = self.AddPool(self.rootPool, "AddIn1")
        newCmd  = self.AddCmd(newPool, "MyCmd1", constants.CMD_NOBREAK, CmdEvtHandler)

    def Delete(self):
        """Main exit point of the command pool."""
        CmdPoolBase.Delete(self)


##
## The Command Event Handler
##
class CmdEvtHandler(CmdEvtHandlerBase):
    """CmdEvtHandler - class derived from convenience class CmdEvtHandlerBase.
    
    An object instance of this class is created on execution of the command with
    which the handler is connected. Its OnExecute function is then called.
    """
    def OnExecute(self, cmd, data, cmdStr):
        """Main entry point of the command event handler on command execution."""
        self.DoSomething()
        
    def DoSomething(self):
        theCADdy      = self.theCADdy
        factory       = theCADdy.Factory2D()
        targetManager = theCADdy.TargetManager()
        types         = theCADdy.DataTypes()
    
        pos1 = types.CADdy2DPosition()
        pos2 = types.CADdy2DPosition()
        
        translate = types.CADdy2DVector()
        phi       = types.CADdyAngleRel()
        matrix    = types.CADdy2DTransform()
        
        pos1.SetCoord(100.0, 100.0)
    
        for a in range(1,360,1):
            phi.Grad = a
    
            translate.X = -pos1.X
            translate.Y = -pos1.Y
            
            matrix.Identity()
            matrix.AppTranslation(translate)
            matrix.AppRotation(phi)
    
            translate.X = pos1.X
            translate.Y = pos1.Y        
            matrix.AppTranslation(translate)
    
            pos2.SetCoord(200,100)
            pos2.Transform(matrix)
    
            line = factory.CreateOp2DStraSeg(pos1,pos2)
            
            AddToParent(line, targetManager.TargetOp2DParent)
    
def main():
    """Main function - called on running the script from the command line.
    
    This function will register the AddIn in the Windows registry as an valid
    COM server.
    """
    RegisterServer(AddIn1)
    
if __name__ == '__main__':
    main()

