##
## AddIn2.py - More lines...
##
"""AddIn sample 2: Debugging example.

This sample series (0-3) of python scripts introduces the way CADdy++ 
AddIns should be written.

An 'AddIn' is a kind of plugin. Actually, it is a registered COM server which 
provides arbitrarily services, which may be meaningful for the CADdy++ user. 

Registration of the AddIn can be performed by just running this script
from the command shell. Deregistration can be performed by running the
script this way: 'AddIn2.py --unregister'

This example shows the generation of a CADdy++ 'Command Pool' 'AddIn2', which
contains a CADdy++ 'Command' 'MyCmd1'. Executing this command will draw just
the same bunch of lines as the example AddIn1.

Starting of the AddIn2 will be performed by running of the associated 
script RunAddIn2.py. After that the command pool 'AddIn2' containing
the command 'MyCmd1' will be established. Command 'AddIn2.MyCmd1' can be
started by associating a CADdy++ tool bar button with it. For that
look into the CADdy++ user documentation or into the CADdy++ application
menu 'View/Symbol Bar'. 

Sample AddIn2 will introduces some of the debugging capabilities of the
CADdy++ Python COM framework. It is most important to get acquainted with
these aids.

Debugging a normal Python application is quite straigthforward. Just start
your favorite Integrated Development Environment 'IDE' and run its debugger
with your application code. The CADdy++ Python framework contains the following
IDE
        - PythonWin IDE,
        
which can be found in the 'CADdyPython' sub menu of your CADdy++ start menu.
Additionally, a standalone remote debugger can be found in its 'Tools' menu.
However, debugging a CADdy++ AddIn is not as simple. This is because the
AddIn itself is a COM server which is started by the CADdy++ application.

Basically, there are two different approaches for debugging a CADdy++ AddIn.
Firstly, using a tracing mechanism provided by the 'PythonWin IDE'.
To use it, do the following:
        - Register your application with '--debug' in the command line
          'AddIn2.py --debug'
        - Start the 'PythonWin' IDE
        - Select 'Tools->Remote Trace Collector'
        - Run the AddIn by executing the 'AddIn2.MyCmd1' command.
Now, every COM call will be prompted to the trace window. All exceptions
will be shown. This way a kind of print debugging can be performed.

Secondly, and probably the more desirable method goes as follows:
        - Register your application the standard way 'AddIn2.py'
        - Insert the following line into the header of the AddIn source
          code:
                from pywin.debugger import *

        - At the point of your source code, where you want to start a
          debug session insert the following function call:
          
                set_trace()
                
          This statement is effectively a hard-coded break point,
          which creates a debugger, and causes it to stop at the statement 
          following the set_trace() call.
        - Run the AddIn by executing the 'AddIn2,MyCmd1' command.
        - Since, the CADdy++ apllication always grabs the input focus on
          mouse movement over its main CAD windows, it may be necessary to
          explicitly switch to the 'PythonWin IDE'.

The second method allows insitu debugging of the AddIn code. 
Of course you may combine the two methods perfectly.

The following code does contain fewer documetation than the previous
examples. If there are still questions, the reader may go back to the
preceding lessons.
"""
from CADdy.AddInServer import *
from CADdy.Utility.Model import AddToParent
from win32com.client import constants

from pywin.debugger import *

##
## The AddIn Server
##
class AddIn2(AddInBase):
    _reg_clsid_  = '{11485D17-2913-4EFD-92EC-FA2F2FAFC9BB}'
    _reg_progid_ = 'CADdyAddIn.PythonAddIn2'

    def __init__(self):
        AddInBase.__init__(self)

    def OnInit(self, theCADdy):
        self.cmdPool = CmdPool(theCADdy)
        
    def OnExit(self, theCADdy):
        self.cmdPool.Delete()


##
## The Command Pool Representation
##
class CmdPool(CmdPoolBase):
    def __init__(self, theCADdy):
        CmdPoolBase.__init__(self, theCADdy)

        newPool = self.AddPool(self.rootPool, "AddIn2")
        newCmd  = self.AddCmd(newPool, "MyCmd1", constants.CMD_NOBREAK, CmdEvtHandler)

    def Delete(self):
        CmdPoolBase.Delete(self)


##
## The Command Event Handler
##
class CmdEvtHandler(CmdEvtHandlerBase):
    def __init__(self):
        """Constructor - just pass all parameters to the base class"""
        CmdEvtHandlerBase.__init__(self)
        
    def OnExecute(self, cmd, data, cmdStr):
        """Envent handler - call worker function"""
        self.DoSomething()
        
    def DoSomething(self):
        """Inside of this function a trace marker is set to force the debugger into action."""
        theCADdy      = self.theCADdy
        factory       = theCADdy.Factory2D()
        targetManager = theCADdy.TargetManager()
        types         = theCADdy.DataTypes()
    
        pos1 = types.CADdy2DPosition()
        pos2 = types.CADdy2DPosition()
        
        ##
        ## Trace marker: This statement is effectively a hard-coded break point,
        ##               which creates a debugger, and causes it to stop at the 
        ##               statement following the set_trace() call.
        ##
        set_trace()
        
        translate = types.CADdy2DVector()
        phi       = types.CADdyAngleRel()
        matrix    = types.CADdy2DTransform()
        
        pos1.SetCoord(100.0, 100.0)
    
        for a in range(1,360,1):
            phi.Grad = a
    
            translate.X = -pos1.X
            translate.Y = -pos1.Y
            
            matrix.Identity()
            matrix.AppTranslation(translate)
            matrix.AppRotation(phi)
    
            translate.X = pos1.X
            translate.Y = pos1.Y        
            matrix.AppTranslation(translate)
    
            pos2.SetCoord(200,100)
            pos2.Transform(matrix)
    
            line = factory.CreateOp2DStraSeg(pos1,pos2)
            
            AddToParent(line, targetManager.TargetOp2DParent)
            
def main():
    """Main function - called on running the script from the command line.
    
    This function will register the AddIn in the Windows registry as an valid
    COM server.
    """
    RegisterServer(AddIn2)
    
if __name__ == '__main__':
    main()

