##
## AddIn3.py - Mouse movement...
##
"""AddIn sample 3: Action example.

This sample series (0-3) of python scripts introduces the way CADdy++ 
AddIns should be written.

An 'AddIn' is a kind of plugin. Actually, it is a registered COM server which 
provides arbitrarily services, which may be meaningful for the CADdy++ user. 

Registration of the AddIn can be performed by just running this script
from the command shell. Deregistration can be performed by running the
script this way: 'AddIn3.py --unregister'

This example shows the generation of a CADdy++ 'Command Pool' 'AddIn3', which
contains a CADdy++ 'Command' 'MyCmd1'. Executing this command will start a
CADdy++ 'Action' which does print the current mouse position into the CADdy++
'Statusblock'. Pressing the left or right mouse buttons, respectively, will 
stop the action.

Starting of the AddIn3 will be performed by running of the associated 
script RunAddIn3.py. After that the command pool 'AddIn3' containing
the command 'MyCmd1' will be established. Command 'AddIn3.MyCmd1' can be
started by associating a CADdy++ tool bar button with it. For that
look into the CADdy++ user documentation or into the CADdy++ application
menu 'View/Symbol Bar'. 

Sample AddIn3 introduces the CADdy++ Action concept and shows the proper
way of writing it with the CADdy++ Python COM framework.

The following code does contain fewer documetation than the previous
examples. If there are still questions, the reader may go back to the
preceding lessons.
"""
from CADdy.AddInServer import *
from CADdy.Utility import Cast
from win32com.client   import constants

## from pywin.debugger import *
## set_trace()

##
## The AddIn Server
##
class AddIn3(AddInBase):
    _reg_clsid_  = '{1B6E2A61-17DC-4587-B91D-BF4DE7436A59}'
    _reg_progid_ = 'CADdyAddIn.PythonAddIn3'

    def __init__(self):
        AddInBase.__init__(self)

    def OnInit(self, theCADdy):
        self.cmdPool = CmdPool(theCADdy)
        
    def OnExit(self, theCADdy):
        self.cmdPool.Delete()


##
## The Command Pool Representation
##
class CmdPool(CmdPoolBase):
    def __init__(self, theCADdy):
        CmdPoolBase.__init__(self, theCADdy)

        newPool = self.AddPool(self.rootPool, "AddIn3")
        newCmd  = self.AddCmd(newPool, "MyCmd1", constants.CMD_BREAK, CmdEvtHandler)

    def Delete(self):
        CmdPoolBase.Delete(self)


##
## The Command Event Handler
##
class CmdEvtHandler(CmdEvtHandlerBase):
    """CmdEvtHandler - class derived from convenience class CmdEvtHandlerBase.
    
    The OnExecute handler of this class just creates an instance of class Action
    defined below and passes all its parameters to the constructor. The action will
    be pushed onto the CADdy++ action stack and stays alive until the user stops it
    (for instance by pressing the right mouse button) or another action replaces it
    on the stack.
    """
    def __init__(self):
        """Constructor - just pass all parameters to the base class"""
        CmdEvtHandlerBase.__init__(self)
        self.myAction = None

    def OnExecute(self, cmd, data, cmdStr):
        """Envent handler - create an action object."""
        self.myAction = Action(cmd, data, cmdStr)


##
## The Action
##
class Action(ActionBase):
    """Action - class derived from convenience class ActionBase.
    
    This class defines a CADdy++ user 'Action'. The base class provides all possible
    action event handlers. The user action may overide some of the event handlers for
    its own purposes.
    """
    def __init__(self, cmd, data, cmdStr):
        """Constructor - just pass all parameters to the base class"""
        ActionBase.__init__(self, cmd, data, cmdStr)
        self.Start()
        
    def OnActionDestroy(self):
        """OnActionDestroy event handler - the proper way for clean up operations
        
        We get the CADdy++ status block from the frame manager and set the the X and Y
        entries to 'undefined'. This will clear the status block lines for these values.
        
        The following return values are possible:
                ACTIONEVENT_CONTINUE: Let the action or filter sitting on below of 'this' action
                                      on the action stack process the same event, i.e. event 
                                      propagation.
                ACTIONEVENT_REPEAT:   Let the same action process the same event again. This might
                                      be desirable after changing the type of the event, that is
                                      calling another event handler of the same action.
                ACTIONEVENT_STOP:     Do not deliver the event to any other handler.
        """
        self.frameManager.StatusBlock.IsUndefinedX = 1
        self.frameManager.StatusBlock.IsUndefinedY = 1
        
        return constants.ACTIONEVENT_CONTINUE
        
    def OnActionBreak(self, event):
        """OnActionBreak event handler - break event.
        
        This event is typically generated by pressing the right mouse button.
        """
        return constants.ACTIONEVENT_CONTINUE
        
    def OnActionPoint2D(self, event):
        """OnActionPoint2D event handler - point event.
        
        This event is typically generated by pressing the left mouse button in a 2D CAD context.
        
        Events carry information, depending on their type. For instance the point
        event carrys the mouse position at the moment the left mouse button was pressed.
        
        Passing the position values to the status block will print them in the status block
        dialog.
        
        This action should stop after pressing the left mouse button. Therefore we change the
        type of the event from ICADdyPoint2DEvent to ICADdyBreakEvent and returning the 
        ACTIONEVENT_CONTINUE flag. 
        """
        self.frameManager.StatusBlock.ValueX = event.XPosition
        self.frameManager.StatusBlock.ValueY = event.YPosition
        
        event = Cast.Event(event)
        event.ChgToBreak()
        
        return constants.ACTIONEVENT_CONTINUE
        
    def OnActionDynamic2D(self, event):
        """OnActionDynamic2D event handler - dynamic event.
        
        This event is generated by moving the mouse pointer in a 2D CAD context.
        
        Events are carrying information, depending on their type. For instance the dynamic
        event carrys the current mouse position.
        
        Passing the position values to the status block will print them in the status block
        dialog.
        """
        self.frameManager.StatusBlock.ValueX = event.XPosition
        self.frameManager.StatusBlock.ValueY = event.YPosition
        
        return constants.ACTIONEVENT_CONTINUE


def main():
    """Main function - called on running the script from the command line.
    
    This function will register the AddIn in the Windows registry as an valid
    COM server.
    """
    RegisterServer(AddIn3)
    
if __name__ == '__main__':
    main()

