##
## Action0.py - Polygon drawing...
##
"""Action sample 0: Polygon drawing

This sample series (0-3) of python scripts introduces the way CADdy++ 
Actions should be written. For details about the CADdy++ AddIn concept,
command pools and commands, go back to Lesson 1.

Registration of the AddIn can be performed by just running this script
from the command shell. Deregistration can be performed by running the
script this way: 'Action0.py --unregister'.

This example generates a command pool 'Action0', containing a command
'MyCmd'. Executing this command will start a CADdy++ 'Action'. This
action will react on moving the mouse pointer inside the 2D CAD MDI
window of the CADdy++ application: After pressing the left mouse 
button line painting starts until either the left mouse button is
pressed again or the right mouse button is pressed.
Pressing the left button again restarts the painting... .
Pressing the right mouse button, in case that draw modus is inactive,
stops the action completely.

This might sound difficult, but it becomes slighly clearer, if we
introduce the notion of the 'action state'.

Our simple action is actually a two state action:
        State 0 : no drawing
                Left button pressed : enter State 1
                Right buton pressed : leave action
        State 1 : drawing
                Left button pressed : enter State 0

Starting of the AddIn will be performed by running of the associated 
script RunAction0.py. After that the command pool 'Action0' containing
the command 'MyCmd' will be established. Command 'Action0.MyCmd' can be
started by associating a CADdy++ tool bar button with it. For that,
look into the CADdy++ user documentation or into the CADdy++ application
menu 'View/Symbol Bar'. 

The following code does contain fewer documetation than the previous
examples. If there are still questions, the reader may go back to the
preceding lessons.
"""
from CADdy.AddInServer import *
from CADdy.Utility.Model import AddToParent
from win32com.client import constants

##
## The AddIn Server
##
class AddIn(AddInBase):
    _reg_clsid_  = '{F7BEC2FF-D5D4-40B2-8638-C24B06845B02}'
    _reg_progid_ = 'CADdyAddIn.PythonAction0'

    def __init__(self):
        AddInBase.__init__(self)

    def OnInit(self, theCADdy):
        self.cmdPool = CmdPool(theCADdy)
        
    def OnExit(self, theCADdy):
        self.cmdPool.Delete()


##
## The Command Pool Representation
##
class CmdPool(CmdPoolBase):
    def __init__(self, theCADdy):
        CmdPoolBase.__init__(self, theCADdy)

        newPool = self.AddPool(self.rootPool, "Action0")
        newCmd  = self.AddCmd(newPool, "MyCmd", constants.CMD_BREAK, CmdEvtHandler)

    def Delete(self):
        CmdPoolBase.Delete(self)


##
## The Command Event Handler
##
class CmdEvtHandler(CmdEvtHandlerBase):
    def __init__(self):
        CmdEvtHandlerBase.__init__(self)
        self.action = None
        
    def OnExecute(self, cmd, data, cmdStr):
        self.action = Action(cmd, data, cmdStr)


##
## The Action
##
class Action(ActionBase):
    """Action - class derived from convenience class ActionBase.
    
    This class defines a CADdy++ user 'Action'. The base class provides all possible
    action event handlers. The user action may overide some of the event handlers for
    its own purposes.
    """
    ##
    ## This action has exactly two possible states
    ##
    _DRAWING_MODE_OFF = 0
    _DRAWING_MODE_ON  = 1
    
    def __init__(self, cmd, data, cmdStr):
        """Constructor - just pass all parameters to the base class"""
        ActionBase.__init__(self, cmd, data, cmdStr)
        
        self.state  = Action._DRAWING_MODE_OFF
        self.parent = self.targetManager.TargetOp2DParent
        
        self.Start()
        
    def OnActionBreak(self, event):
        if self.state == Action._DRAWING_MODE_OFF:
            return constants.ACTIONEVENT_CONTINUE
        else:
            self.state = Action._DRAWING_MODE_OFF
            return constants.ACTIONEVENT_STOP
        
    def OnActionPoint2D(self, event):
        if self.state == Action._DRAWING_MODE_OFF:
            self.state = Action._DRAWING_MODE_ON
            self.PosStart = event.Position2D
        else:
            self.state = Action._DRAWING_MODE_OFF
        
        return constants.ACTIONEVENT_CONTINUE
        
    def OnActionDynamic2D(self, event):
        if self.state == Action._DRAWING_MODE_ON:
            PosEnd = event.Position2D
            line = self.factory2D.CreateOp2DStraSeg(self.PosStart, PosEnd)
        
            AddToParent(line, self.targetManager.TargetOp2DParent)

            self.PosStart = PosEnd

        return constants.ACTIONEVENT_CONTINUE


def main():
    """Main function - called on running the script from the command line.
    
    This function will register the AddIn in the Windows registry as an valid
    COM server.
    """
    RegisterServer(AddIn)
    
if __name__ == '__main__':
    main()

