##
## Action1.py - Polygon drawing...
##
"""Action sample 1: Polygon drawing

This sample series (0-3) of python scripts introduces the way CADdy++ 
Actions should be written. For details about the CADdy++ AddIn concept,
command pools and commands, go back to Lesson 1.

Registration of the AddIn can be performed by just running this script
from the command shell. Deregistration can be performed by running the
script this way: 'Action1.py --unregister'.

This example generates a command pool 'Action1', containing a command
'MyCmd'. Executing this command will start a CADdy++ 'Action'. This
action will allow the construction of polygon lines. The action is
programmed after the following state definition:

        Start
            After starting, the action immediatly switches into the 
            'define_first_point' state. 
        State 'define_first_point'
            The state's responsibility is the definition of the first
            polygon point. By pressing the left mouse button a
            point event is generated which carrys the position of
            the first point. The action remembers this first position.
            The point event triggers the transition into the 
            'define_next_point' state. However, pressing the right 
            mouse button generates a break event. This event stops
            the action immediatly.
            Since this action should handle the 2D as well as the 3D
            graphic context, it must implement the associated event
            handlers, respectively. They are provided by the base
            action.
            Changing the CADdy++ working sketchboard, should also
            stop the action. Therefore the action implements the
            event handler for sketchboard change.
        State define_next_point
            The state's responsibility is the definition of the next
            polygon points. I.e. it defines a polygon segment from
            the remembered last position to the position provided
            by the point event triggered by pressing the left mouse
            button. At the end of the point event, the action
            passes over to the 'define_first_point' state.
            Pressing the right mouse button generates a break event
            and the action switches back into the 'define_first_point'
            state, but without the definition of a polygon segment.
            The sketchboard change event likewise stops the action.
            However, this state must also handle dynamic events,
            generated by the mouse movement in the 2D and 3D CAD 
            context, since the user should have some response to the 
            construction process. For that, we will draw the polygon
            segment dynamically to the current mouse pointer position.
            A dynamic event will not force the action into another
            state, however.
            
The states are implemented by Python classes StateFirstPoint and
StateNextPoint. Instances of these two classes are administred by
the action object, which will be created as always on command
execution.

Starting of the AddIn will be performed by running of the associated 
script RunAction1.py. After that the command pool 'Action1' containing
the command 'MyCmd' will be established. Command 'Action1.MyCmd' can be
started by associating a CADdy++ tool bar button with it. For that,
look into the CADdy++ user documentation or into the CADdy++ application
menu 'View/Symbol Bar'. 

The following code does contain fewer documetation than the previous
examples. If there are still questions, the reader may go back to the
preceding lessons.
"""
from CADdy.AddInServer import *
from CADdy.Utility import Cast
from CADdy.Utility.Model import AddToParent
from win32com.client   import constants

## from pywin.debugger import *
## set_trace()

##
## The AddIn Server
##
class AddIn(AddInBase):
    _reg_clsid_  = '{73480F5E-989B-4D84-A723-90CE4D9726DC}'
    _reg_progid_ = 'CADdyAddIn.PythonAction1'

    def __init__(self):
        AddInBase.__init__(self)

    def OnInit(self, theCADdy):
        self.cmdPool = CmdPool(theCADdy)
        
    def OnExit(self, theCADdy):
        self.cmdPool.Delete()


##
## The Command Pool Representation
##
class CmdPool(CmdPoolBase):
    def __init__(self, theCADdy):
        CmdPoolBase.__init__(self, theCADdy)

        newPool = self.AddPool(self.rootPool, "Action1")
        newCmd  = self.AddCmd(newPool, "MyCmd", constants.CMD_BREAK, CmdEvtHandler)

    def Delete(self):
        CmdPoolBase.Delete(self)


##
## The Command Event Handler
##
class CmdEvtHandler(CmdEvtHandlerBase):
    def __init__(self):
        CmdEvtHandlerBase.__init__(self)
        self.action = None
        
    def OnExecute(self, cmd, data, cmdStr):
        #import rpdb2; rpdb2.start_embedded_debugger("MyTest")
        self.action = Action(cmd, data, cmdStr)


##
## The Action
##
class Action(ActionBase):
    """Action - class derived from convenience class ActionBase.
    
    This class defines a CADdy++ user 'Action'. The base class provides all
    possible action event handlers. The user action may overide some of the
    event handlers for its own tasks. Actually, the state classes provides
    the event handlers used for the specific state.
    """
    def __init__(self, cmd, data, cmdStr):
        """Constructor - just pass all parameters to the base class.
        
        The action uses a point definition filter. A filter preprocesses
        events send to an action. The point definition filter provides
        special services like end point catching, middle point catching
        and so on.
        The line member attribute will be used by the different state
        instances.
        The state member attribut will be used for dispatching of the
        action events.
        """
        ActionBase.__init__(self, cmd, data, cmdStr)

        self.state = StateFirstPoint(self)
        self.line = None
        
        self.Start()
        
        self.pointDefFilter = self.frameManager.UsePointDefFilter(0)

    def OnActionDestroy(self):
        """OnActionDestroy - clean up operation."""
        self.pointDefFilter = None
        self.Pop()
        return constants.ACTIONEVENT_CONTINUE

    def OnActionBreak(self, event):
        """OnActionBreak - handled by the current state instance."""
        return self.state.OnActionBreak(event)

    def OnActionPoint2D(self, event):
        """OnActionBreak - handled by the current state instance."""
        return self.state.OnActionPoint2D(event)

    def OnActionDynamic2D(self, event):
        """OnActionBreak - handled by the current state instance."""
        return self.state.OnActionDynamic2D(event)

    def OnActionSketchBoardChange(self, event):
        """OnActionSketchBoardChange - default behaviour: stop action"""
        self.state = None
        return constants.ACTIONEVENT_CONTINUE

##
## StateFirstPoint
##
class StateFirstPoint:
    def __init__(self, action):
        """Constructor - State instances remembers the action they are used by.
        
        Typically, a state is associated by a user prompt, stating the action
        requested from the user.
        """
        self.action = action
        self.theCADdy = self.action.theCADdy

        self.action.Push("Define first point")

    def OnActionBreak(self, event):
        """OnActionBreak - stop the action."""
        self.action.state = None
        return constants.ACTIONEVENT_CONTINUE

    def OnActionPoint2D(self, event):
        """OnActionPoint2D - A new line has to be created.
        
        Initially, the line is created from the first point to the first point, i.e.
        with zero length.
        
        The handler then switches into the dynamic drawing mode. The following dynamic
        event will dynamically update the end point of the just defined line segment.
        
        Finally, the handler transits into the 'StateNextPoint' state.
        """
        posFirst = event.Position2D
        
        self.action.line = self.action.factory2D.CreateOp2DStraSeg(posFirst, posFirst)

        AddToParent(self.action.line, self.action.targetManager.TargetOp2DParent)

        element2D = Cast.Element2D(self.action.line)
        element2D.InitDynamicMode()

        self.action.Pop()
        self.action.state = StateNextPoint(self.action)
        
        return constants.ACTIONEVENT_STOP

    def OnActionDynamic2D(self, event):
        """OnActionDynamic2D - this state does not perform any dynamic operations."""
        return constants.ACTIONEVENT_STOP

##
## StateNextPoint
##
class StateNextPoint:
    def __init__(self, action):
        """Constructor - State instances remembers the action they are used by.
        
        Typically, a state is associated by a user prompt, stating the action
        requested from the user.
        """
        self.action = action
        self.theCADdy = self.action.theCADdy
        
        self.action.Push("Define next point")

    def OnActionBreak(self, event):
        """OnActionBreak - transition into state 'StateFirstPoint'.
        
        This handler has to abort the current line drawing and to clean up the
        aquired resources.
        """
        if self.action.line is not None:
            element2D = Cast.Element2D(self.action.line)
            element2D.AbortDynamicMode()
            object = Cast.Object(self.action.line)
            object.Delete()
            self.action.line = None

        self.action.Pop()
        self.action.state = StateFirstPoint(self.action)
        return constants.ACTIONEVENT_STOP

    def OnActionPoint2D(self, event):
        """OnActionPoint2D - a line segment is completly defined.
        
        This handler's duty is the exit of the dynamic draw mode and the setting of
        the final line end point. A CADdy++ 'undo/redo switchstate' is set after the
        line has been completed. This way, a transaction point is defined which can
        be reached by the user performing undo or redo operations, respectively.
        
        Additionally, a new line segment definition is started, followed by entering
        of the dynamic mode.
        """
        posNext = event.Position2D
        
        if self.action.line is not None:
            self.action.line.PositionEnd = posNext
            
            element2D = Cast.Element2D(self.action.line)
            element2D.ExitDynamicMode()
        
            self.action.undoRedoManager.SwitchState()
            
            self.action.line = self.action.factory2D.CreateOp2DStraSeg(posNext, posNext)

            AddToParent(self.action.line, self.action.targetManager.TargetOp2DParent)

            element2D = Cast.Element2D(self.action.line)
            element2D.InitDynamicMode()
            
        return constants.ACTIONEVENT_STOP

    def OnActionDynamic2D(self, event):
        """OnActionDynamic2D - dynamic line end point adapting."""
        posNext = event.Position2D
        
        if self.action.line is not None:
            self.action.line.PositionEnd = posNext
        
        return constants.ACTIONEVENT_STOP


def main():
    """Main function - called on running the script from the command line.
    
    This function will register the AddIn in the Windows registry as an valid
    COM server.
    """
    RegisterServer(AddIn)
    
if __name__ == '__main__':
    main()

