"""Variante2: Boden

Erzeugt die Geometrie fuer einen kreisfoermigen Boden mit Radius 'R', der in
Streifen der Breite 'b' aufgeteilt ist. Die Mitte ist in 'cx,cy'.

Die Bemassung wird hinzugefuegt."""
import math
from CADdy.Application   import GetApplic
from CADdy.Utility.Model import AddToParent
from win32com.client     import constants

theCADdy = None

################################################################################

def CreateGeometry(cx, cy, R, b, target):

    factory       = theCADdy.Factory2D()
    typesManager  = theCADdy.DataTypesManager()

    # Zur Erstellung der Kreissegmente wird spaeter u.a. die Mittelposition benoetigt
    center = typesManager.Create2DPosition(cx, cy)

    # Die von links nach rechts sortierten Eckpunkte der
    # Bleche kommen in eine Liste von je drei Punkten:
    #       Die Schnittpunkte auf der Mittelachse.
    #       die Schnittpunkt der Streifenkanten auf dem Kreis oben und unten
    #       Das sind die Loesungen von (x-cx)**2 +(y-cy)**2 = R**2
    
    # Die am weitesten links gelegene Position ist (cx-R, cy)
    pos = typesManager.Create2DPosition(cx-R, cy)
    
    positions = []
    positions.append( (pos,pos,pos) )
    
    # Zuerst kommt ein Streifen mit der Restbreite R % b, dann Streifen der Breite b.
    d = -R + (R % b)
    
    while d < R:
        h = math.sqrt(R*R - d*d)
        
        posO = typesManager.Create2DPosition(cx+d, cy+h)
        posM = typesManager.Create2DPosition(cx+d, cy)
        posU = typesManager.Create2DPosition(cx+d, cy-h)
        positions.append( (posO, posM, posU) )
        d += b
        
    # Die am weitesten rechts gelegene Position ist (cx+R, cy).
    pos = typesManager.Create2DPosition(cx+R, cy)
    positions.append( (pos, pos, pos) )

    # Beim Erzeugen der Berandungen zugleich die Linien einsammeln,
    # die spaeter fuer die Bemassung benoetigt werden
    linesO = []
    linesU = []

    # Die Liste der vorgefertigten Positionen wird durchlaufen, um
    # die Begrenzungslinien zu erzeugen.
    posOL, posML, posUL = positions[0]; i = 0
    for posOR, posMR, posUR in positions[1:]:
        i+=1
        groupO = factory.CreateOp2DGroup("Blech %d" % i)
        AddToParent(groupO,target)
        
        i+=1
        groupU = factory.CreateOp2DGroup("Blech %d" % i)
        AddToParent(groupU,target)

        basisO = factory.CreateOp2DStraSeg(posMR, posML)
        lineLO = factory.CreateOp2DStraSeg(posML, posOL)
        lineRO = factory.CreateOp2DStraSeg(posOR, posMR)
    
        basisU = factory.CreateOp2DStraSeg(posMR, posML)
        lineLU = factory.CreateOp2DStraSeg(posML, posUL)
        lineRU = factory.CreateOp2DStraSeg(posUR, posMR)
    
        vec1 = posOL.Difference(center)
        vec2 = posOR.Difference(center)
    
        angle1, len = vec1.GetPolar()
        angle2, len = vec2.GetPolar()
    
        diff = angle1.Difference(angle2)
        circleO = factory.CreateOp2DCirSeg(center, R, angle2, diff)
        
        angle2.Grad = -angle2.Grad
        diff.Grad   = -diff.Grad
        circleU = factory.CreateOp2DCirSeg(center, R, angle2, diff)

        AddToParent(lineLO,  groupO)
        AddToParent(circleO, groupO)
        AddToParent(lineRO,  groupO)
        AddToParent(basisO,  groupO)

        AddToParent(lineLU,  groupU)
        AddToParent(circleU, groupU)
        AddToParent(lineRU,  groupU)
        AddToParent(basisU,  groupU)

        linesO.append(lineLO)
        linesU.append(lineLU)

        posOL, posML, posUL = posOR, posMR, posUR

    linesO.append(lineRO)
    linesU.append(lineRU)

    return linesO,linesU
    
################################################################################

def CreateChainDimension(lines, target, x, y, vx, vy, nc):
    """Erzeugt Bemassungen.

    lines   Liste von Strecken
    target  Teilzeichnung oder Gruppe, wo eingefuegt werden soll
    x,y     Position auf der Masslinie
    vx,vy   Richtung der Masshilfslinie
    nc      True fuer NC-Bemassung, False fuer normale Kettenbemassung"""
    targetManager    = theCADdy.TargetManager()
    typesManager     = theCADdy.DataTypesManager()
    dimensionManager = theCADdy.DimensionManager()
    # 'pos' Parameter in Bemassungsdaten; ist hier ohne Wirkung.
    # Er waere notwendig zum Identifizieren in  zusammengesetzter Geometrie wie z.B. 
    # Komponenten.
    pos = typesManager.Create2DPosition(0.0, 0.0)

    # Bemassungsdaten sammeln
    data = theCADdy.Dimension().CADdyOpDimChainData()
    for line in lines:
        data.Add(line, pos, 0.0, True) # am Startpunkt

    pos.SetCoord(x, y) # Position der Massline
    vec = typesManager.Create2DUnitVector(constants.AXIS2D_Y) # Richtung der Masshilfslinien
    vec.SetCoord(vx, vy)

    # assoziative Kettenbemassung erzeugen:
    if (nc):
        annotation = dimensionManager.CreateOpDimChainNCAssociative(data, pos, vec)
    else:
        annotation = dimensionManager.CreateOpDimChainAssociative(data, pos, vec)

    AddToParent(annotation,target)

################################################################################

def main():
    cx,cy = 250.0, 150.0
    R = 100.0
    b = 35.0

    # Der CADdy-API-Einstieg wird ueberall benoetigt, darum einmal global holen
    global theCADdy
    theCADdy = GetApplic()

    # Die aktuelle Teilzeichnung ermitteln
    targetManager = theCADdy.TargetManager()
    target = targetManager.TargetOp2DSketchboard

    # Unterhalb der aktuellen Teilzeichnung werden fuer jede Blechkontur Gruppen
    # angelegt.
    linesO, linesU = CreateGeometry(cx,cy, R, b, target)

    # oben Kettenbemassung
    CreateChainDimension(linesO, target, cx, cy + 1.5*R, 0.0, 1.0, False)

    # unten NC-Bemassung
    CreateChainDimension(linesU, target, cx, cy - 1.5*R, 0.0, 1.0, True)

if __name__ == '__main__':
    main()

